/*
 *    Copyright (C) 1998 Nikos Mavroyanopoulos
 *    Copyright (C) 1999,2000 Sascha Schumman, Nikos Mavroyanopoulos
 *
 *    This library is free software; you can redistribute it and/or modify it 
 *    under the terms of the GNU Library General Public License as published 
 *    by the Free Software Foundation; either version 2 of the License, or 
 *    (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this library; if not, write to the
 *    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *    Boston, MA 02111-1307, USA.
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "mhash.h"

void mhash_bzero(void *s, int n);

static char hexconvtab[] = "0123456789ABCDEF";

static char *bin2hex(const unsigned char *old, const size_t oldlen,
		     size_t * newlen)
{
	unsigned char *new = NULL;
	int i, j;

	new = (char *) malloc(oldlen * 2 * sizeof(char) + 1);
	if (new == NULL)
		return new;

	for (i = j = 0; i < oldlen; i++) {
		new[j++] = hexconvtab[old[i] >> 4];
		new[j++] = hexconvtab[old[i] & 15];
	}
	new[j] = '\0';

	if (newlen)
		*newlen = oldlen * 2 * sizeof(char);

	return new;
}

static int test_hash(hashid HashId, char *input_data, char *awaited_hash)
{
	size_t bsize;
	unsigned char *data;
	size_t data_len;
	char *str;
	size_t str_len;
	MHASH td;
	int ret = 0;

	data_len = strlen(input_data);

	if ((str = mhash_get_hash_name(HashId))==NULL) 
		return 1;

	bsize = mhash_get_block_size(HashId);
	if (!bsize)
		return 1;

	data = (unsigned char *) malloc(data_len + 1);
	if (data == NULL)
		return 1;
	mhash_bzero(data, data_len + 1);

	if (data_len)
		strncpy(data, input_data, data_len);

#ifdef TEST_DEBUG
	printf("Input text: %s\n", (char *) data);
#endif

	td = mhash_init(HashId);
	mhash(td, data, data_len);
	data = mhash_end(td);
	str = bin2hex(data, bsize, &str_len);
	free(data);

#ifdef TEST_DEBUG
	printf("Result : %s\nAwaited: %s\n", str, awaited_hash);
#endif

	if (strcmp(str, awaited_hash) != 0) {
		printf("Test failed !\n");
		ret = 1;
	}
		
	free(str);
	
	return ret;
}

int main(int argc, char **argv)
{
	int ret;

	printf("\nThis utility tests all the hash functions of libmhash.\n\n");

	printf("Testing MD5...\n");
	if (test_hash(1, "", "D41D8CD98F00B204E9800998ECF8427E"))
		return 1;
	if (test_hash(1, "a", "0CC175B9C0F1B6A831C399E269772661"))
		return 1;
	if (test_hash(1, "abc", "900150983CD24FB0D6963F7D28E17F72"))
		return 1;
	if (test_hash(1, "message digest", "F96B697D7CB7938D525A2F31AAF161D0"))
		return 1;
	if (test_hash(1, "abcdefghijklmnopqrstuvwxyz", "C3FCD3D76192E4007DFB496CCA67E13B"))
		return 1;
	if (test_hash(1, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", "D174AB98D277D9F5A5611C2C9F419D9F"))
		return 1;
	if (test_hash(1, "12345678901234567890123456789012345678901234567890123456789012345678901234567890", "57EDF4A22BE3C955AC49DA2E2107B67A"))
		return 1;
	printf("MD5 test successful !\n\n");
	
	printf("Testing SHA1...\n");
	if (test_hash(2, "abc", "A9993E364706816ABA3E25717850C26C9CD0D89D"))
		return 1;
	if (test_hash(2, "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq", "84983E441C3BD26EBAAE4AA1F95129E5E54670F1"))
		return 1;
	printf("SHA1 test successful !\n\n");
	
	printf("Testing HAVAL256...\n");
	if (test_hash(3, "abcdefghijklmnopqrstuvwxyz", "72FAD4BDE1DA8C8332FB60561A780E7F504F21547B98686824FC33FC796AFA76"))
		return 1;
	if (test_hash(3, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", "899397D96489281E9E76D5E65ABAB751F312E06C06C07C9C1D42ABD31BB6A404"))
		return 1;
	printf("HAVAL256 test successful !\n\n");

	printf("Testing HAVAL224...\n");
	if (test_hash(10, "0123456789", "EE345C97A58190BF0F38BF7CE890231AA5FCF9862BF8E7BEBBF76789"))
		return 1;
	printf("HAVAL224 test successful !\n\n");

	printf("Testing HAVAL192...\n");
	if (test_hash(11, "HAVAL", "8DA26DDAB4317B392B22B638998FE65B0FBE4610D345CF89"))
		return 1;
	printf("HAVAL192 test successful !\n\n");

	printf("Testing HAVAL160...\n");
	if (test_hash(12, "a", "4DA08F514A7275DBC4CECE4A347385983983A830"))
		return 1;
	printf("HAVAL160 test successful !\n\n");

	printf("Testing HAVAL128...\n");
	if (test_hash(13, "", "C68F39913F901F3DDF44C707357A7D70"))
		return 1;
	printf("HAVAL128 test successful !\n\n");

	printf("Testing RIPEMD160...\n");
	if (test_hash(5, "", "9C1185A5C5E9FC54612808977EE8F548B2258D31"))
		return 1;
	if (test_hash(5, "a", "0BDC9D2D256B3EE9DAAE347BE6F4DC835A467FFE"))
		return 1;
	if (test_hash(5, "abc", "8EB208F7E05D987A9B044A8E98C6B087F15A0BFC"))
		return 1;
	if (test_hash(5, "message digest", "5D0689EF49D2FAE572B881B123A85FFA21595F36"))
		return 1;
	if (test_hash(5, "abcdefghijklmnopqrstuvwxyz", "F71C27109C692C1B56BBDCEB5B9D2865B3708DBC"))
		return 1;
	if (test_hash(5, "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq", "12A053384A9C0C88E405A06C27DCF49ADA62EB2B"))
		return 1;
	if (test_hash(5, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", "B0E20B6E3116640286ED3A87A5713079B21F5189"))
		return 1;
	printf("RIPEMD160 test successful !\n\n");

	printf("Testing GOST...\n");
	if (test_hash(8, "This is message, length=32 bytes", "B1C466D37519B82E8319819FF32595E047A28CB6F83EFF1C6916A815A637FFFA"))
		return 1;
	if (test_hash(8, "Suppose the original message has length = 50 bytes", "471ABA57A60A770D3A76130635C1FBEA4EF14DE51F78B4AE57DD893B62F55208"))
		return 1;
	printf("GOST test successful !\n\n");

#if 1
	/* The TIGER test program displays values in the wrong endian */
	printf("Testing TIGER...\n");
	if (test_hash(7, "", "3293AC630C13F0245F92BBB1766E16167A4E58492DDE73F3"))
		return 1;
	if (test_hash(7, "abc", "2AAB1484E8C158F2BFB8C5FF41B57A525129131C957B5F93"))
		return 1;
	if (test_hash(7, "Tiger", "DD00230799F5009FEC6DEBC838BB6A27DF2B9D6F110C7937"))
		return 1;
	if (test_hash(7, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-", "F71C8583902AFB879EDFE610F82C0D4786A3A534504486B5"))
		return 1;
	if (test_hash(7, "ABCDEFGHIJKLMNOPQRSTUVWXYZ=abcdefghijklmnopqrstuvwxyz+0123456789", "48CEEB6308B87D46E95D656112CDF18D97915F9765658957"))
		return 1;
	if (test_hash(7, "Tiger - A Fast New Hash Function, by Ross Anderson and Eli Biham", "8A866829040A410C729AD23F5ADA711603B3CDD357E4C15E"))
		return 1;
	if (test_hash(7, "Tiger - A Fast New Hash Function, by Ross Anderson and Eli Biham, proceedings of Fast Software Encryption 3, Cambridge.", "CE55A6AFD591F5EBAC547FF84F89227F9331DAB0B611C889"))
		return 1;
	if (test_hash(7, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-", "C54034E5B43EB8005848A7E0AE6AAC76E4FF590AE715FD25"))
		return 1;
	printf("TIGER test successful !\n\n");
#endif

	printf("\nAll tests were succesful !\n\n");
	
	return 0;
}
